from dataclasses import dataclass
from enum import Enum
from typing import Optional

import plotly.graph_objects as go
import polars as pl

from blackfish.plotting.gaussian_broadening import apply_gaussian_filter
from blackfish.plotting.templates import plotly_template

SpectrumType = Enum("SpectrumType", "SOC_TDDFT TDDFT IR")


class Spectrum:
    def __init__(self, df: pl.DataFrame, spectrum_type: SpectrumType):
        self.df = df
        self.spectrum_type = spectrum_type
        self.fig = self.create_figure()

    def create_figure(self):
        match self.spectrum_type:
            case SpectrumType.SOC_TDDFT:
                return self._create_soc_tddft_figure()
            case SpectrumType.TDDFT:
                return self._create_tddft_figure()
            case SpectrumType.IR:
                return self._create_ir_figure()
            case _:
                raise ValueError("Invalid spectrum type")

    def _create_soc_tddft_figure(self):
        fig = go.Figure()
        fig.update_layout(
            template=plotly_template,
            width=1024,
            height=480,
            title="SOC-TDDFT Spectrum",
            xaxis_title="Energy [1/cm]",
            yaxis_title="Rel. Intensity",
        )
        fig.add_trace(SOCTDDFTBarTrace(self.df, name="SOC Absorption").to_plotly())
        fig.add_trace(SOCTDDFTSimulatedTrace(self.df, name="Simulated").to_plotly())
        fig.update_xaxes(autorange="reversed", tickformat="d")
        return fig

    def show(self, **kwargs):
        """Display the spectrum"""
        self.fig.show(**kwargs)


@dataclass
class BaseTrace:
    df: pl.DataFrame
    x: Optional[str] = None
    y: Optional[str] = None
    name: Optional[str] = None
    color: Optional[str] = None

    def to_plotly(self):
        """Convert to plotly trace"""
        raise NotImplementedError


@dataclass
class SOCTDDFTBarTrace(BaseTrace):
    def to_plotly(self) -> go.Bar:
        x_col = self.x or "energy_cm"
        y_col = self.y or "rel_intensity"
        color_col = self.color or "mult"
        return go.Bar(
            x=self.df[x_col],
            y=self.df[y_col],
            customdata=self.df[["state", "mult", "energy_ev", "wavelength_nm"]],
            name=self.name,
            width=100,
            marker=dict(
                color=self.df[color_col],
                colorbar=dict(title=color_col, titleside="right", tickvals=[1, 2, 3]),
            ),
            hovertemplate="State: <b>%{customdata[0]:8d}</b><br>"
            + "Mult: %{customdata[1]:8d}<br>"
            + "Energy: %{x:8.1f} cm<sup>-1</sup><br>"
            + "Energy: %{customdata[2]:8.2f} eV<br>"
            + "Rel. Intensity: %{y:8.2f}<br>"
            + "Wavelength: %{customdata[3]:8d} nm<br>",
        )


@dataclass
class SOCTDDFTSimulatedTrace(BaseTrace):
    def to_plotly(self, fwhm: int = 2000) -> go.Scatter:
        x_col = self.x or "energy_cm"
        y_col = self.y or "rel_intensity"
        color = self.color or "black"
        broadened_df = apply_gaussian_filter(self.df, x_col, y_col, fwhm, 10)
        return go.Scatter(
            x=broadened_df[x_col],
            y=broadened_df[y_col],
            customdata=[10e6 / i for i in broadened_df[x_col]],
            mode="lines",
            line=dict(color=color),
            name=f"FWHM = {fwhm:4d}",
            hovertemplate="Wavelength: %{customdata:.8d} nm",
        )



# ruff: noqa

from blackfish.plotting.spectrum import Spectrum, SpectrumType
from blackfish.plotting.templates import plotly_template
from blackfish.plotting.soc_absorption_spectrum_plot import soc_absorption_spectrum_plot



import plotly.graph_objects as go

plotly_template = go.layout.Template(
    dict(
        layout=go.Layout(
            title=dict(font_family="verdana", font_size=20, x=0.5, xanchor="center"),
            width=1280,
            height=480,
            margin=dict(l=75, r=75, t=100, b=75),
            plot_bgcolor="whitesmoke",
            xaxis=dict(
                mirror=True,
                showline=True,
                ticks="inside",
                zeroline=False,
                showgrid=False,
                linewidth=2,
            ),
            yaxis=dict(
                mirror=True,
                showline=True,
                ticks="inside",
                zeroline=True,
                zerolinewidth=2,
                showgrid=False,
                linewidth=2,
            ),
            showlegend=True,
            legend=dict(
                x=0.99,
                y=0.99,
                xanchor="right",
                yanchor="top",
                bordercolor="grey",
                borderwidth=1,
                bgcolor="white",
            ),
            barcornerradius="40%",
            # hovermode="closest",
            hovermode="x",
            hoverlabel=dict(
                bgcolor="white",
                font_size=14,
                # font_family="Rockwell"
                font_family="Arial",
            ),
        )
    )
)



from pathlib import Path

import plotly.graph_objects as go

from blackfish.plotting import plotly_template

ROOT = Path(__file__).parent


def test_plotly_template_works():
    template = plotly_template
    fig = go.Figure()
    fig.update_layout(template=template)
    assert isinstance(fig, go.Figure)



import marimo

__generated_with = "0.10.9"
app = marimo.App(width="medium")


@app.cell
def _():
    import marimo as mo
    import blackfish as bf
    return bf, mo


@app.cell
def _(bf):
    orca = bf.orca.ORCA("/Users/freddy/Documents/Projects/group10_triazole_azides/Group10_Triazole_Azide_Complexes/rerun/calculations/singlet/azide/tddft/pd/tddft.out")
    return (orca,)


@app.cell
def _(orca):
    orca.soc_absorption_spectrum
    return


@app.cell
def _(mo, orca):
    mo.ui.data_explorer(orca.soc_absorption_spectrum)
    return


@app.cell
def _(plot):
    plot
    return


@app.cell
def _():
    return


if __name__ == "__main__":
    app.run()



from pathlib import Path

from blackfish.parsing import (
    ir_spectrum,
    nacme,
    roots,
    soc_absorption_spectrum,
    soc_states,
)

ROOT = Path(__file__).parent


def test_parsing_ir_spectrum():
    df = ir_spectrum(ROOT / "data/ir_spectrum.txt")
    assert len(df) == 237
    assert df.columns == [
        "mode",
        "frequency_cm",
        "epsilon",
        "intensity",
        "t2",
        "tx",
        "ty",
        "tz",
        "rel_intensity",
    ]


def test_parsing_nacme():
    df = nacme(ROOT / "data/nacme.txt")
    assert len(df) == 81
    assert df.columns == [
        "id",
        "symbol",
        "x",
        "y",
        "z",
        "magnitude",
    ]


def test_parsing_roots():
    df = roots(ROOT / "data/roots.txt")
    assert len(df) == 180
    assert df.columns == [
        "root",
        "mult",
        "donor",
        "acceptor",
        "weight",
        "energy_cm",
    ]


def test_parsing_soc_absorption_spectrum():
    df = soc_absorption_spectrum(ROOT / "data/soc_absorption_spectrum.txt")
    assert len(df) == 64
    assert df.columns == [
        "state",
        "mult",
        "energy_ev",
        "energy_cm",
        "wavelength_nm",
        "osc_strength",
        "d2",
        "dx",
        "dy",
        "dz",
        "rel_intensity",
    ]


def test_parsing_soc_states():
    df = soc_states(ROOT / "data/soc_states.txt")
    assert len(df) == 9
    assert df.columns == [
        "state",
        "spin",
        "root",
        "weight",
        "energy_cm",
    ]
